<?php
// ---------------------------------------------------------------------------
// $Id$
// ---------------------------------------------------------------------------
// OpenRat Content Management System
// Copyright (C) 2002-2004 Jan Dankert, cms@jandankert.de
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


define('OR_NOTICE_OK'   ,'ok'     );
define('OR_NOTICE_WARN' ,'warning');
define('OR_NOTICE_ERROR','error'  );

/**
 * Eltern-Klasse fuer alle Actions.
 *
 * Diese Klasse stellt grundlegende action-uebergreifende Methoden
 * bereit.
 * Dient als Ueberklasse fuer alle abgeleiteten Action-Klassen in
 * diesem Package bzw. Verzeichnis.
 *
 * @author Jan Dankert
 * @version $Revision$
 * @package openrat.actions
 * @abstract 
 */
class Action
{
	var $db;
	var $templateVars = Array();
	var $actionName;
	var $subActionName;
	var $actionClassName;

	var $writable;
	var $publishing;
	var $actionConfig;
	
	/**
	 * Aktuell angemeldeter Benutzer.<br>
	 * Wird ind er Funktion "init()" gesetzt.
	 *
	 * @var Object Benutzer
	 */
	var $currentUser;


	/**
	 * Wird durch das Controller-Skript (do.php) nach der Kontruierung des Objektes aufgerufen.
	 * So koennen Unterklassen ihren eigenen Kontruktor besitzen, ohne den Superkontruktor
	 * (=diese Funktion) aufrufen zu mssen.
	 */
	function init()
	{
		global $conf;
		$this->writable    = !$conf['security']['readonly' ];
		$this->publishing  = !$conf['security']['nopublish'];
		$this->currentUser = Session::getUser();
		
		$this->templateVars['errors' ] = array();
		$this->templateVars['notices'] = array();
	}


	/**
	 * Liest eine Session-Variable
	 *
	 * @param String $varName Schlssel
	 * @return mixed
	 */
	function getSessionVar( $varName )
	{
		global $SESS;

		if	( !isset($SESS[ $varName ]) )
			return '';
		else	return $SESS[ $varName ];
	}


	/**
	 * Setzt eine Session-Variable
	 *
	 * @param Sring $varName Schlssel
	 * @param mixed $value Inhalt
	 * @return mixed
	 */
	function setSessionVar( $varName,$value )
	{
		global $SESS;

		$SESS[ $varName ] = $value;
	}


	/**
	 * Ermittelt den Inhalt der gewnschten Request-Variablen.
	 * Falls nicht vorhanden, wird "" zurckgegeben.
	 *
	 * @param String $varName Schlssel
	 * @return String Inhalt
	 */
	function getRequestVar( $varName )
	{
		global $REQ;

		if	( !isset($REQ[ $varName ]) )
			return '';
		else	return $REQ[ $varName ];
	}


	/**
	 * Ermittelt, ob der aktuelle Request eine Variable mit dem
	 * angegebenen Namen enthlt.
	 *
	 * @param String $varName Schlssel
	 * @return boolean true, falls vorhanden.
	 */
	function hasRequestVar( $varName )
	{
		global $REQ;

		return( !empty($REQ[$varName]) );
	}


	/**
	 * Ermittelt die aktuelle Id aus dem Request.<br>
	 * Um welche ID es sich handelt, ist abhngig von der Action.
	 *
	 * @return Integer
	 */
	function getRequestId()
	{
		return intval( $this->getRequestVar( REQ_PARAM_ID ) );
	}



	/**
	 * Setzt eine Variable fr die Oberflche.
	 *
	 * @param String $varName Schlssel
	 * @param Mixed $value
	 */
	function setTemplateVar( $varName,$value )
	{
		$this->templateVars[ $varName ] = $value;
	}


	/**
	 * Setzt eine Liste von Variablen fr die Oberflche.
	 *
	 * @param Array $varList Assoziatives Array
	 */
	function setTemplateVars( $varList )
	{
		foreach( $varList as $name=>$value )
		{
			$this->setTemplateVar( $name,$value );
		}
	}


	/**
	 * Fgt einen Validierungsfehler hinzu.
	 *
	 * @param String $name Name des validierten Eingabefeldes
	 * @param String Textschlssel der Fehlermeldung (optional)
	 */
	function addValidationError( $name,$message="COMMON_VALIDATION_EROR" )
	{
		if	( !empty($message) )
			$this->addNotice('','',$message,OR_NOTICE_ERROR);

		$this->templateVars['errors'][] = $name;
	}

	
	/**
	 * Fgt ein Meldung hinzu.
	 *
	 * @param String $type Typ des Objektes, zu dem diese Meldung gehrt.
	 * @param String $name Name des Objektes, zu dem diese Meldung gehrt.
	 * @param String $text Textschlssel der Fehlermeldung (optional)
	 * @param String $status Einer der Werte OR_NOTICE_(OK|WARN|ERROR)
	 * @param Array  $vars Variablen fr den Textschlssel
	 * @param Array $log Weitere Hinweistexte fr diese Meldung.
	 */
	function addNotice( $type,$name,$text,$status=OR_NOTICE_OK,$vars=array(),$log=array() )
	{
		if	( !is_array($log))
			$log = array($log);

		if	( !is_array($vars))
			$vars = array($vars);

		$this->templateVars['notices'][] = array('type'=>$type,
                                                 'name'=>$name,
		                                         'text'=>lang('NOTICE_'.$text,$vars),
		                                         'log'=>$log,
		                                         'status'=>$status);
	}


	
	/**
	 * @deprecated siehe #addNotice
	 * @param unknown_type $title
	 * @param unknown_type $add_info
	 */
	function message( $title='ERROR',$add_info='' )
	{
		Logger::warn( 'creating error message, info='.$add_info );

		$this->setTemplateVar( 'title',lang( $title         ) );
		$this->setTemplateVar( 'text' ,lang( $title.'_DESC' ) );
		$this->setTemplateVar( 'info' ,$add_info );
		
		$this->forward('message');
	}


	/**
	 * Ausgabe des Templates.<br>
	 * <br>
	 * Erst hier soll die Ausgabe auf die Standardausgabe, also die
	 * Ausgabe fr den Browser, starten.<br>
	 * <br>
	 *
	 * @param String Relativer Dateiname des Templates im Format "<action>/<subaction>".
	 */
	function forward( $tplName="" )
	{
		if	( isset($this->actionConfig[$this->subActionName]['direct']) )
			exit;

		$this->setMenu();
		
		$tplName = $this->actionName.'/'.$this->subActionName;

			
		if	( isset($this->actionConfig[$this->subActionName]['action']) )
			$tplName = $this->actionConfig[$this->subActionName]['action'].'/'.$this->subActionName;

		if	(isset($this->actionConfig[$this->subActionName]['alias']))
			$tplName = (method_exists(new ObjectAction(),$this->subActionName)?'object':$this->actionName).'/'.$this->actionConfig[$this->subActionName]['alias'];
			
		if	(isset($this->actionConfig[$this->subActionName]['target']))
			$targetSubActionName = $this->actionConfig[$this->subActionName]['target'];

		if	( isset($this->actionConfig[$this->subActionName]['menu']))
			$windowTitle = 'menu_title_'.$this->actionName.'_'.$this->actionConfig[$this->subActionName]['menu'];

		global $conf;
		global $REQ;
		global $PHP_SELF;
		global $HTTP_SERVER_VARS;
		global $image_dir;
		       
		$tplName = str_replace( '_','/',$tplName );
		
		$tplFileName = $tplName.'.tpl.'.PHP_EXT;
		$conf_php = PHP_EXT;
	
		// ?bertragen der Array-Variablen in den aktuellen Kontext
		//
		extract( $this->templateVars );

		// ?bertragen der Array-Variablen in den aktuellen Kontext
		//
		
		if	( count($errors)>0 )
			extract( $REQ );
	
		// Setzen einiger Standard-Variablen
		//
		$tpl_dir    = OR_THEMES_DIR.$conf['interface']['theme'].'/pages/html/';
		$image_dir  = OR_THEMES_EXT_DIR.$conf['interface']['theme'].'/images/';
	
		$user = Session::getUser();
		
		if	( strpos($conf['interface']['style']['extend'],'/')===false )
			$root_stylesheet = OR_THEMES_EXT_DIR.$conf['interface']['theme'].'/css/'.$conf['interface']['style']['extend'].'.css';
		else
			$root_stylesheet = $style['extend'];
			
		if	( !is_object($user) )
			$user_stylesheet = OR_THEMES_EXT_DIR.$conf['interface']['theme'].'/css/'.$conf['interface']['style']['default'].'.css';
		else
			$user_stylesheet = OR_THEMES_EXT_DIR.$conf['interface']['theme'].'/css/'.$user->style.'.css';
		
		$self = $HTTP_SERVER_VARS['PHP_SELF'];
	
		$tplFileName = str_replace( '_','/',$tplFileName );

		if	( !empty($conf['interface']['override_title']) )
			$cms_title = $conf['interface']['override_title'];
		else
			$cms_title = OR_TITLE.' '.OR_VERSION;

		$showDuration = $conf['interface']['show_duration'];

		$subActionName = $this->subActionName;
		$actionName    = $this->actionName;
		$requestId     = $this->getRequestId();
		
		if	( $conf['theme']['compiler']['enable'] )
		{
			$te = new TemplateEngine();
			$te->compile( $tplName );
			unset($te);
		}

		// Einbinden des Templates
		require( $tpl_dir.$tplFileName );
		
		exit;
	}
	
	
	/**
	 * Ruft eine weitere Subaction auf.
	 *
	 * @param String $subActionName Name der nchsten Subaction. Es muss eine Methode mit diesem Namen geben.
	 */
	function callSubAction( $subActionName )
	{
		if	( in_array($this->actionName,array('page','file','link','folder')) )
			Session::setSubaction( $subActionName );

		$this->subActionName = $subActionName;		

		Logger::trace("next subaction is '$subActionName'");
		
		$this->$subActionName();
	}


	/**
	 * Ermitteln, ob Benutzer Administratorrechte besitzt
	 * @return Boolean TRUE, falls der Benutzer ein Administrator ist.
	 */
	function userIsAdmin()
	{
		$user = Session::getUser();
		return $user->isAdmin;
	}



	/**
	 * Ermitteln des Benutzerobjektes aus der Session
	 * @return User
	 */
	function getUserFromSession()
	{
		return Session::getUser();
	}


	
	/**
	 * Benutzen eines sog. "Conditional GET".
	 *
	 * Diese Funktion setzt einen "Last-Modified"-HTTP-Header.
	 * Ist der Inhalt der Seite nicht neuer, so wird der Inhalt
	 * der Seite nicht ausgegeben, sondern nur HTTP-Status 304
	 * ("304 not modified") gesetzt.
	 * Der Rest der Seite muss dann nicht mehr erzeugt werden,
	 * wodurch die Performance stark erhoeht werden kann.
	 * Ggf. kann das Benutzen dieses Mechanismus zu unerwnschten
	 * Effekten fhren, dann muss "conditional GET" in der
	 * Konfiguration deaktiviert werden.
	 *
	 * Credits: Danke an Charles Miller
	 * @see http://fishbowl.pastiche.org/2002/10/21/http_conditional_get_for_rss_hackers
	 *
	 * Gefunden auf:
	 * @see http://simon.incutio.com/archive/2003/04/23/conditionalGet
	 *
	 * @param Timestamp Letztes Aenderungsdatum dieser Seite
	 */
	function lastModified( $time )
	{
		$user = Session::getUser();
		if	( $user->loginDate > $time )
			// Falls Benutzer-Login nach letzter nderung.
			// Zweck: Nach einem Login sollte mind. 1x jede Seite neu geladen werden, dies
			// Ist z.B. nach einer Style-nderung durch den Benutzer notwendig.
			// Falls aus Versehen doch einmal zuviel gecacht wurde, kann man das durch ein
			// Neu-Login beheben. 
			$time = $user->loginDate;

		// Conditional-Get eingeschaltet?
		global $conf;
		if	( ! $conf['cache']['conditional_get'] )
			return;

		$lastModified = substr(date('r',$time-date('Z')),0,-5).'GMT';
		$etag         = '"'.md5($lastModified).'"';

		// Header senden
		header('Last-Modified: '.$lastModified );
		header('ETag: '         .$etag          );
		
		// Die vom Interpreter sonst automatisch gesetzten
		// Header uebersteuern
		header('Cache-Control: must-revalidate');
		header('Pragma:');

		// See if the client has provided the required headers
		$if_modified_since = isset($_SERVER['HTTP_IF_MODIFIED_SINCE']) ? stripslashes($_SERVER['HTTP_IF_MODIFIED_SINCE']) : false;
		$if_none_match     = isset($_SERVER['HTTP_IF_NONE_MATCH']    ) ? stripslashes($_SERVER['HTTP_IF_NONE_MATCH']    ) :	false;

		if	( !$if_modified_since && !$if_none_match )
			return;

		// At least one of the headers is there - check them
		if	( $if_none_match && $if_none_match != $etag )
			return; // etag is there but doesn't match


		if	( $if_modified_since && $if_modified_since != $lastModified )
			return; // if-modified-since is there but doesn't match

		// Der entfernte Browser bzw. Proxy holt die Seite nun aus seinem Cache 
		header('HTTP/1.0 304 Not Modified');
		exit;  // Sofortiges Skript-Ende
	}



	/**
	 * @param max Anzahl der Sekunden, die die Seite im Browsercache bleiben darf
	 */
	function maxAge( $max=3600 ) 
	{
		// Die Header "Last-Modified" und "ETag" wurden bereits in der
		// Methode "lastModified()" gesetzt.
		
		header('Expires: '.substr(date('r',time()-date('Z')+$max),0,-5).'GMT' );
		header('Pragma: '); // 'Pragma' ist Bullshit und
		                    // wird von den meisten Browsern ignoriert.
		header('Cache-Control: public, max-age='.$max.", s-maxage=".$max);
	}	
	
	
	
	function setMenu()
	{
		if	(!isset($this->actionConfig[$this->subActionName]['menu']))
			return;
		$windowMenu = array();
		$name       = $this->actionConfig[$this->subActionName]['menu'];
		$menuList   = explode(',',$this->actionConfig['menu'][$name]);

		if	( isset($this->actionConfig[$this->subActionName]['action']))
			$actionName = $this->actionConfig[$this->subActionName]['action'];
		else
			$actionName = $this->subActionName;
		
		foreach( $menuList as $menuName )
		{
			if	( isset($this->actionConfig[$menuName]['alias']) )
				$menuText = 'menu_'.$this->actionName.'_'.$this->actionConfig[$menuName]['alias'];
			else
				$menuText = 'menu_'.$this->actionName.'_'.$menuName;
			
			
			$menuKey = 'accesskey_window_'.$menuName;
			
//			Logger::trace("testing menu $menuName");
			$menuEntry = array('subaction'=>$menuName,
                               'text'     =>$menuText,
                               'title'    =>$menuText.'_DESC',
                               'key'      =>$menuKey );
                               
			if	( $this->checkMenu($menuName) )
				$menuEntry['url'] = Html::url($actionName,$menuName,$this->getRequestId());
				
			$windowMenu[] = $menuEntry;
		}
		$this->setTemplateVar('windowMenu',$windowMenu);
	}
	
	
	
	/**
	 * Ermittelt, ob der Menpunkt aktiv ist.
	 * Ob ein Menpunkt als aktiv angezeigt werden soll, steht meist erst zur Laufzeit fest.
	 * <br>
	 * Diese Methode kann von den Unterklassen berschrieben werden.
	 * Falls diese Methode nicht berschrieben wird, sind alle Menpunkte aktiv.
	 *
	 * @param String $name Logischer Name des Menpunktes
	 * @return boolean TRUE, wenn Menpunkt aktiv ist.
	 */
	function checkMenu( $name )
	{
		// Standard: Alle Menpunkt sind aktiv.
		return true;
	}
}

?>